/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTBindingEnd.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTLayerConnector.h>
#include <RTLocalBindingDescriptor.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

void RTActor::initializeAll( void )
{
    const RTActor_class         * info      = getActorData();
    const RTComponentDescriptor * component = info->components;
    int                           i;

    for( i = info->numComponents; --i >= 0; ++component )
    {
	RTActorRef * ref = RTObjectOf( RTActorRef, component->offset, this );

	if( ref->isFixed() != 0 )
	    ref->initializeAll();
    }

    RTController                   * controller = context();
    const RTPortDescriptor         * port       = info->ports;
    const RTLocalBindingDescriptor * binding    = info->bindings;

    for( i = info->numBindings; --i >= 0; ++binding )
    {
	int          replication = binding->replication;
	RTBindingEnd aEnd;
	RTBindingEnd zEnd;

	aEnd.port  = RTObjectOf( RTProtocol,
				 port[ binding->aPortIndex ].offset, this );
	aEnd.index = 0;
	zEnd.port  = RTObjectOf( RTProtocol,
				 port[ binding->zPortIndex ].offset, this );
	zEnd.index = 0;

	RTAssert( replication != 0 );
	do
	{
	    for(;;)
	    {
		if( aEnd.index >= aEnd.port->size() )
		{
		    RTDiag::panic( "local bind a exhausted" );
		    //lint -unreachable
		    return;
		}
		if( aEnd.port->isBoundAt( aEnd.index ) == 0 )
		    break;
		++aEnd.index;
	    }

	    for(;;)
	    {
		if( zEnd.index >= zEnd.port->size() )
		{
		    RTDiag::panic( "local bind z exhausted" );
		    //lint -unreachable
		    return;
		}
		if( zEnd.port->isBoundAt( zEnd.index ) == 0 )
		    break;
		++zEnd.index;
	    }

	    RTLayerConnector::bind( controller, aEnd, zEnd );
	}
	while( --replication != 0 );
    }

    for( i = info->numPorts; --i >= 0; ++port )
    {
	if( port->replication <= 0 ) // excluded?
	    continue;

	const char * regName;

	if( (regName = port->layerName) == (const char *)0 &&
	    (regName = port->name)      == (const char *)0 )
	{
	    continue;
	}

	int properties = port->properties;

	if( ( properties & ( RTPortDescriptor::MaskKind
			   | RTPortDescriptor::MaskRegisterAuto ) )
			!= ( RTPortDescriptor::KindUnwired
			   | RTPortDescriptor::MaskRegisterAuto ) )
	    continue;

	RTProtocol * instance = RTObjectOf( RTProtocol, port->offset, this );

	if( ( properties & RTPortDescriptor::MaskRegisterPublished ) == 0 )
	    (void)instance->registerSAP( regName, RTProtocol::Automatic );
	else
	    (void)instance->registerSPP( regName, RTProtocol::Automatic );
    }

    for( port = info->ports, i = info->numPorts; --i >= 0; ++port )
    {
	RTProtocol * instance = RTObjectOf( RTProtocol, port->offset, this );

	if( ( port->properties & RTPortDescriptor::MaskKind )
			      == RTPortDescriptor::KindSpecial )
	    continue;

	instance->notifyInit( controller );
    }
}
