/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTBindingEnd.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTInterfaceDescriptor.h>
#include <RTLayerConnector.h>
#include <RTRelayDescriptor.h>
#include <RTRelayPort.h>


int RTActorRef::bind_ports( RTActor * actor, int index, RTActorRef * importer )
{
    RTAssert( actor != (RTActor *)0 );

    RTActor                     * myOwner   = getOwner();
    const RTActorRef            * actorHome = actor->getReference();
    const RTActorClass          * type      = actor->getActorData()->type;
    const int                     numRelays = type->numRelays;
    const int                     numIfaces = _info->numInterfaces;
    int                           relayId;
    const RTRelayDescriptor     * relayDesc;
    RTRelayPort                 * relay;
    const RTInterfaceDescriptor * iface;

    RTController * controller = myOwner->context();	

	
    for( relayId   = 0,
	 relayDesc = type->relays,
	 relay     = actor->relays,
	 iface     = _info->interfaces;

	 relayId < numIfaces;

	 ++relayId, ++relayDesc, ++relay, ++iface )
    {
	int count = iface->replication;

	if( count == 0 )
	    continue;

	if( relayId >= numRelays )
	{
	    // error: too few relay ports
	}
	else if( relay->reserve( actorHome, relayId, count,
				 relayDesc->replication, importer ) == 0 )
	{
	    // error: reservation failed
	}
	else
	{
	    continue;
	}

	// back-track undoing the reservations made to this point

	while( --relayId >= 0 )
	{
	    --relayDesc;
	    --relay;
	    --iface;
	    relay->unreserve( myOwner, relayId, relayDesc->replication,
			      importer,controller);
	}
	return 0;
    }

    
    RTBindingEnd   end1;
    RTBindingEnd   end2;

    for( relayId   = 0,
	 relayDesc = type->relays,
	 relay     = actor->relays,
	 iface     = _info->interfaces;

	 relayId < numIfaces;

	 ++relayId, ++relayDesc, ++relay, ++iface )
    {
	int count = iface->replication;

	if( count == 0 )
	    continue;

	int relaySize = relayDesc->replication;
	int relayBase = count * index;

	for( int relayIndex = 0; relayIndex < count; ++relayIndex )
	{
	    if( relay->followIn( end1, actor, relayId,
				 relayIndex, relaySize, importer ) == 0 )
		continue;

	    if( myOwner->_followOutV( end2, getId(),
				      relayId, relayBase + relayIndex ) == 0 )
		continue;

	    RTLayerConnector::lock();
	    RTLayerConnector::bind( controller, end1, end2,
				    RTLayerConnector::CheckedAlready );
	    RTLayerConnector::unlock();
	}
    }

    return 1;
}
