/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

int RTActorRef::destroy( RTActor * actor )
{
    // This is the initiator context
    RTController * controller = context();

    if( actor == (RTActor *)0 )
    {
	controller->setError( RTController::badActor,
			      RTController::FrameDestroy,
			      this,
			      (const void *)0 );
	return 0;
    }

    int index = actor->getIndex();

    if( index < 0 || size() <= index || (*this)[ index ] != actor )
    {
	controller->setError( RTController::badIndex,
			      RTController::FrameDestroy,
			      this,
			      (const void *)0 );
	return 0;
    }

    // disable binding notifications: we will soon purge any rtUnbound messages
    const RTActor_class    * info  = actor->getActorData();
    const RTPortDescriptor * port  = info->ports;
    int                      count = info->numPorts;

    for( port = info->ports, count = info->numPorts; --count >= 0; ++port )
    {
	if( ( port->properties & RTPortDescriptor::MaskKind )
			      != RTPortDescriptor::KindSpecial )
	{
	    RTProtocol * end = RTObjectOf( RTProtocol, port->offset, actor );

	    end->bindingNotification( 0 );
	}
    }

    // deport actor from all of the places that imported it
    actor->removeAllImports(controller);
   
#if USE_THREADS && RTFRAME_THREAD_SAFE
    lock();
#endif
    actor->_unbind((RTActorRef *)0,controller);
#if USE_THREADS && RTFRAME_THREAD_SAFE
    unlock();
#endif

    remove( index );

    controller->destroy( actor, getOwner() );

    return 1;
}
