/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTCachedString.h>
#include <RTDiag.h>
#include <RTDictionary.h>
#include <RTMemoryUtil.h>

unsigned                RTCachedString::_size;
unsigned                RTCachedString::_used;
RTCachedString::Block * RTCachedString::_blocks;
RTCachedString::Entry** RTCachedString::_contents = (RTCachedString::Entry**)0;

static RTCachedString::Block * new_Block( RTCachedString::Block * next )
{
    RTCachedString::Block * block = new RTCachedString::Block;
    RTCachedString::Entry * entry;
    int                     count;

    block->next = next;

    for( entry = &block->entry[ 0 ], count = RTCachedString::Block::Size;
				     --count >= 0;
	 ++entry )
    {
	entry->value     = (char *)0;
	entry->use_count = 0U;
    }

    return block;
}

RTCachedString::Entry * RTCachedString::add( const char * value )
{
    if( value == (const char *)0 )
	return (Entry *)0;

    if( _contents == (Entry **)0 )
    {
	Block * block = new_Block( (Block *)0 );
	Entry * entry;
	int     index;

	_size     = Block::Size;
	_used     = 0U;
	_contents = new Entry * [ (unsigned)Block::Size ];
	_blocks   = block;

	for( entry = &block->entry[ 0 ], index = 0;
					 index < Block::Size;
	     ++entry,                    ++index )
	{
	    _contents[ index ] = entry;
	}
    }

    for(;;) // loop back (at most once) if we need to expand the table
    {
	unsigned   index   = RTDictionary::hash( value, _size );
	unsigned   tests   = 0U;
	Entry    * deleted = (Entry *)0;
	Entry    * entry;

	for(;;)
	{
	    entry = _contents[ index ];

	    if( entry->value == (char *)0 )
	    {
		// this slot has never been used
		if( deleted != (Entry *)0 )
		    entry = deleted; // use a deleted entry, if we saw one
		break;
	    }

	    if( entry->use_count == 0U )
	    {
		if( deleted == (Entry *)0 )
		    deleted = entry;
	    }
	    else if( RTMemoryUtil::strcmp( entry->value, value ) == 0 )
	    {
		// the value already exists
		entry->use_count += 1U;
		return entry;
	    }

	    if( ++tests == _size )
	    {
		// we looked everywhere; use the first deleted entry, if any
		entry = deleted;
		break;
	    }

	    if( ++index == _size )
		index = 0U;
	}

	if( entry != (Entry *)0 )
	{
	    entry->value     = RTMemoryUtil::strdup( value );
	    entry->use_count = 1U;

	    ++_used;

	    return entry;
	}

	// the table needs to be expanded

	unsigned    old_size     = _size;
	Entry    ** old_contents = _contents;

	_size   <<= 1U;
	_contents = new Entry * [ _size ];

	for( index = 0U; index < _size; ++index )
	    _contents[ index ] = (Entry *)0;

	for( unsigned old_index = 0U; old_index < old_size; ++old_index )
	{
	    entry = old_contents[ old_index ];
	    index = RTDictionary::hash( entry->value, _size );

	    // search for an empty slot
	    while( _contents[ index ] != (Entry *)0 )
		if( ++index == _size )
		    index = 0U;

	    _contents[ index ] = entry;
	}

	delete [] old_contents;

	// allocate blocks for the half that is still empty
	{
	    unsigned count = 0U;

	    for( index = 0U; index < _size; ++index )
	    {
		if( _contents[ index ] == (Entry *)0 )
		{
		    if( count == 0U )
		    {
			_blocks = new_Block( _blocks );
			entry = &_blocks->entry[ 0 ];
			count = Block::Size;
		    }
		    _contents[ index ] = entry++; //lint !e794
		    --count;
		}
	    }

	    RTAssert( count == 0U );
	}
    }
}
