/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActor_class.h>
#include <RTBindingEnd.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTRelayDescriptor.h>
#include <RTRelayPort.h>

int RTProtocol::isIndexTo( int index, RTActor * actor ) const
{
    if( actor == (RTActor *)0 )
	return 0; // invalid actor id

    if( index < 0 || _info->replication <= index )
	return 0; // index out of range

    End * end = &_ends[ index ];

    if( end->state != End::Unbound && end->remote->getOwner() == actor )
	return 1;

    const RTActorClass      * type      = actor->getActorData()->type;
    const int                 numRelays = type->numRelays;
    int                       relayId   = 0;
    const RTRelayDescriptor * relay     = type->relays;
    RTBindingEnd              peer;

    for( ; relayId < numRelays; ++relayId, ++relay )
    {
	int relaySize = relay->replication;

	for( int repIndex = 0; repIndex < relaySize; ++repIndex )
	{
	    if( actor->_followOut( peer, relayId, repIndex ) != 0 )
		if( peer.port == this && peer.index == index )
		    return 1;
	    if( actor->_followInV( peer, relayId, repIndex ) != 0 )
		if( peer.port == this && peer.index == index )
		    return 1;
	}
    }

    return 0;
}
