/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTObserver_h__
#define __RTObserver_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTElasticArray_h__
#include <RTElasticArray.h>
#endif

class RTDaemon;
class RTDaemonInfo;
class RTDebugger;
class RTEventInfo;
class RTTcpSocket;

class RTObserver
{
protected:
    enum LastFlowCmd { FlowNone, FlowPause, FlowResume };

    RTDebugger     * debugger;
    RTTcpSocket    * socket;
    int              observerId;
    RTDaemon       * daemon;
    RTElasticArray   daemons;
    RTDaemon       * superDaemon;

    int              bufSize;
    int              bufUsed;
    char           * inBuffer;

    int              haltByProbe;
    int              resumeToRun;
    LastFlowCmd      lastFlowCmd;

public:
		     RTObserver( RTDebugger  * theDebugger,
				 RTTcpSocket * theSocket,
				 int           theObsId,
				 const char  * initBuf = (const char *)0,
				 int           initLen = 0 );

    virtual          ~RTObserver( void );
    int              setDaemon( int daemonNr );

    void             cmdCommand      ( const char * commandString );
    void             cmdCreateDaemon ( int daemonNr );
    void             cmdDestroyDaemon( int daemonNr );

    int              addInfo( RTDaemonInfo * theInfo );
    int              delInfo( RTDaemonInfo * theInfo );

    int              prepareEvent(  RTEventInfo & eventInfo );
    void             dispatchEvent( RTEventInfo & eventInfo );

    virtual int      evtChain(        int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtPresence(     int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtReference(    int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtMessageIn(    int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtMessageOut(   int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtMessageOutIn( int                  daemonId,
				      const RTDaemonInfo & sndDaemonInfo,
				      const RTDaemonInfo & recDaemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtRtsStatus(    int                  daemonId,
				      const RTEventInfo  & eventInfo );

    virtual int      evtVariable(     int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      evtHistory(      int                  daemonId,
				      const RTDaemonInfo & daemonInfo,
				      const RTEventInfo  & eventInfo );

    virtual int      receiveData ( void );
    virtual int      transmitData( void );

    int              readData( char * buf, int buflen );
    int              sendData( const char * buf, int buflen = 0 );

private:
    // unavailable methods
    RTObserver( const RTObserver & );
    RTObserver & operator=( const RTObserver & );
}; //lint !e1712

#endif // __RTObserver_h__
