/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTWinCE.h>

void RTConsole::outputString( const char * string )
{
    RECT affected;

    // We begin using buffer coordinates.
    affected.top    = cursor.row;
    affected.bottom = cursor.row;
    affected.left   = cursor.col;
    affected.right  = cursor.col - 1;

    int scroll_x = 0;
    int scroll_y = 0;
    int spaces   = 0;

    for(;;)
    {
	char ch;

	if( spaces > 0 )
	    --spaces, ch = ' ';
	else if( (ch = *string++) == '\0' )
	    break;

	switch( ch )
	{
	case '\b':
	    if( cursor.col > 0 )
	    {
		cursor.col -= 1;
		if( affected.left > cursor.col )
		    affected.left = cursor.col;
	    }
	    break;

	case '\t':
	    spaces = 8 - ( cursor.col & 7 );
	    break;

	case '\n':
	    spaces = buffer.cols - cursor.col;
	    break;

	case '\r':
	    // Ignore CR: it is normally followed immediately by a LF.
	    break;

	default:
	    buffer.data[ cursor.row * buffer.cols + cursor.col++ ] = (TCHAR)ch;
	    if( affected.right < cursor.col )
		affected.right = cursor.col;
	    if( cursor.col >= buffer.cols )
	    {
		cursor.col    = 0;
		affected.left = 0;
		if( cursor.row < buffer.rows - 1 )
		    ++cursor.row;
		else
		{
		    TCHAR * line1 = buffer.data;
		    TCHAR * line2 = &line1[ buffer.cols ];
		    int     i;

		    for( i = ( buffer.rows - 1 ) * buffer.cols; --i >= 0; )
			*line1++ = *line2++;
		    for( i = buffer.cols; --i >= 0; )
			*line1++ = TEXT( ' ' );
		    --scroll_y;
		}
	    }
	    break;
	}
    }

    if( affected.left < screen.left )
    {
	scroll_x    = screen.left - affected.left;
	screen.left = affected.left;
    }

    int screen_right = screen.left + screen.cols - 1;

    if( cursor.col > screen_right )
    {
	scroll_x     = screen_right - cursor.col;
	screen.left -= scroll_x;
    }

    int screen_bottom = screen.top - 1;

    if( screen.rows > buffer.rows )
	screen_bottom += buffer.rows;
    else
	screen_bottom += screen.rows;

    if( scroll_y == 0 )
    {
	if( cursor.row > screen_bottom )
	    scroll_y = screen_bottom - cursor.row;
	else if( affected.top < screen.top )
	    scroll_y = screen.top - affected.top;
	screen.top -= scroll_y;
    }
    else
    {
	int below_screen = screen_bottom - cursor.row;

	scroll_y     -= below_screen;
	screen.top   -= below_screen;
    }

    if( scroll_x != 0 || scroll_y != 0 )
    {
	RECT       rc;
	SCROLLINFO si;

	GetClientRect( window, &rc );
	makeCaret( CaretHidden );

	UpdateWindow( window );
	ScrollWindowEx( window,
			scroll_x * character.width,
			scroll_y * character.height,
			(const RECT *)0, &rc,
			(HRGN)0, (RECT *)0, SW_INVALIDATE );

	if( scroll_y < 0 )
	    affected.top += scroll_y;

	if( scroll_x != 0 )
	{
	    // adjust the horizontal scrollbar
	    si.cbSize = sizeof( si );
	    si.fMask  = SIF_RANGE | SIF_POS | SIF_PAGE;
	    si.nMin   = 0;
	    si.nMax   = buffer.cols - 1;
	    si.nPos   = screen.left;
	    si.nPage  = screen.cols;
	    SetScrollInfo( window, SB_HORZ, &si, TRUE );
	}

	if( scroll_y != 0 )
	{
	    // adjust the vertical scrollbar
	    si.cbSize = sizeof( si );
	    si.fMask  = SIF_RANGE | SIF_POS | SIF_PAGE;
	    si.nMin   = 0;
	    si.nMax   = buffer.rows - 1;
	    si.nPos   = screen.top;
	    si.nPage  = screen.rows;
	    SetScrollInfo( window, SB_VERT, &si, TRUE );
	}

	makeCaret( CaretVisible );
    }

    affected.bottom = cursor.row + 1;

    // Translate to screen (character) coordinates.
    OffsetRect( &affected, - screen.left, - screen.top );

    // Clip to screen extent.
    if( affected.top    < 0 )
	affected.top    = 0;
    if( affected.left   < 0 )
	affected.left   = 0;
    if( affected.bottom > screen.rows )
	affected.bottom = screen.rows;
    if( affected.right  > screen.cols )
	affected.right  = screen.cols;

    if( affected.top < affected.bottom && affected.left < affected.right )
    {
	// Translate to screen (pixel) coordinates.
	affected.top    *= character.height;
	affected.bottom *= character.height;
	affected.left   *= character.width;
	affected.right  *= character.width;

	InvalidateRect( window, &affected, 0 );
    }
}
